@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,7 +19,6 @@
 
 """Functions related to ad blocking."""
 
-import io
 import os.path
 import functools
 import posixpath
@@ -34,7 +33,7 @@ from qutebrowser.api import (cmdutils, hook, config, message, downloads,
                              interceptor, apitypes, qtutils)
 
 
-logger = logging.getLogger('misc')
+logger = logging.getLogger('network')
 _host_blocker = typing.cast('HostBlocker', None)
 
 
@@ -129,8 +128,8 @@ class HostBlocker:
         """Block the given request if necessary."""
         if self._is_blocked(request_url=info.request_url,
                             first_party_url=info.first_party_url):
-            logger.info("Request to {} blocked by host blocker."
-                        .format(info.request_url.host()))
+            logger.debug("Request to {} blocked by host blocker."
+                         .format(info.request_url.host()))
             info.block()
 
     def _read_hosts_line(self, raw_line: bytes) -> typing.Set[str]:
@@ -247,7 +246,7 @@ class HostBlocker:
         self._in_progress.append(download)
         self._on_download_finished(download)
 
-    def _merge_file(self, byte_io: io.BytesIO) -> None:
+    def _merge_file(self, byte_io: typing.IO[bytes]) -> None:
         """Read and merge host files.
 
         Args:
@@ -303,6 +302,9 @@ class HostBlocker:
         self._in_progress.remove(download)
         if download.successful:
             self._done_count += 1
+            assert not isinstance(download.fileobj,
+                                  downloads.UnsupportedAttribute)
+            assert download.fileobj is not None
             try:
                 self._merge_file(download.fileobj)
             finally:
