@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -24,16 +24,18 @@
 import functools
 import os
 import traceback
+import typing
 
 from PyQt5.QtCore import QUrl
-from PyQt5.QtWidgets import QApplication  # pylint: disable=unused-import
+from PyQt5.QtWidgets import QApplication
 
 from qutebrowser.browser import qutescheme
 from qutebrowser.utils import log, objreg, usertypes, message, debug, utils
+from qutebrowser.keyinput import modeman
 from qutebrowser.commands import runners
 from qutebrowser.api import cmdutils
 from qutebrowser.misc import (  # pylint: disable=unused-import
-    consolewidget, debugcachestats, objects)
+    consolewidget, debugcachestats, objects, miscwidgets)
 from qutebrowser.utils.version import pastebin_version
 from qutebrowser.qt import sip
 
@@ -50,8 +52,7 @@ def later(ms: int, command: str, win_id: int) -> None:
     if ms < 0:
         raise cmdutils.CommandError("I can't run something in the past!")
     commandrunner = runners.CommandRunner(win_id)
-    app = objreg.get('app')
-    timer = usertypes.Timer(name='later', parent=app)
+    timer = usertypes.Timer(name='later', parent=QApplication.instance())
     try:
         timer.setSingleShot(True)
         try:
@@ -107,44 +108,43 @@ def run_with_count(count_arg: int, command: str, win_id: int,
 
 
 @cmdutils.register()
-def clear_messages():
+def clear_messages() -> None:
     """Clear all message notifications."""
     message.global_bridge.clear_messages.emit()
 
 
 @cmdutils.register(debug=True)
-def debug_all_objects():
+def debug_all_objects() -> None:
     """Print a list of  all objects to the debug log."""
     s = debug.get_all_objects()
     log.misc.debug(s)
 
 
 @cmdutils.register(debug=True)
-def debug_cache_stats():
+def debug_cache_stats() -> None:
     """Print LRU cache stats."""
     debugcachestats.debug_cache_stats()
 
 
 @cmdutils.register(debug=True)
-def debug_console():
+def debug_console() -> None:
     """Show the debugging console."""
-    try:
-        con_widget = objreg.get('debug-console')
-    except KeyError:
+    if consolewidget.console_widget is None:
         log.misc.debug('initializing debug console')
-        con_widget = consolewidget.ConsoleWidget()
-        objreg.register('debug-console', con_widget)
+        consolewidget.init()
 
-    if con_widget.isVisible():
+    assert consolewidget.console_widget is not None
+
+    if consolewidget.console_widget.isVisible():
         log.misc.debug('hiding debug console')
-        con_widget.hide()
+        consolewidget.console_widget.hide()
     else:
         log.misc.debug('showing debug console')
-        con_widget.show()
+        consolewidget.console_widget.show()
 
 
 @cmdutils.register(maxsplit=0, debug=True, no_cmd_split=True)
-def debug_pyeval(s, file=False, quiet=False):
+def debug_pyeval(s: str, file: bool = False, quiet: bool = False) -> None:
     """Evaluate a python string and display the results as a web page.
 
     Args:
@@ -182,7 +182,7 @@ def debug_pyeval(s, file=False, quiet=False):
 
 
 @cmdutils.register(debug=True)
-def debug_set_fake_clipboard(s=None):
+def debug_set_fake_clipboard(s: str = None) -> None:
     """Put data into the fake clipboard and enable logging, used for tests.
 
     Args:
@@ -197,13 +197,13 @@ def debug_set_fake_clipboard(s=None):
 @cmdutils.register()
 @cmdutils.argument('win_id', value=cmdutils.Value.win_id)
 @cmdutils.argument('count', value=cmdutils.Value.count)
-def repeat_command(win_id, count=None):
+def repeat_command(win_id: int, count: int = None) -> None:
     """Repeat the last executed command.
 
     Args:
         count: Which count to pass the command.
     """
-    mode_manager = objreg.get('mode-manager', scope='window', window=win_id)
+    mode_manager = modeman.instance(win_id)
     if mode_manager.mode not in runners.last_command:
         raise cmdutils.CommandError("You didn't do anything yet.")
     cmd = runners.last_command[mode_manager.mode]
@@ -224,24 +224,6 @@ def log_capacity(capacity: int) -> None:
     log.ram_handler.change_log_capacity(capacity)
 
 
-@cmdutils.register(debug=True)
-@cmdutils.argument('level', choices=sorted(
-    (level.lower() for level in log.LOG_LEVELS),
-    key=lambda e: log.LOG_LEVELS[e.upper()]))
-def debug_log_level(level: str) -> None:
-    """Change the log level for console logging.
-
-    Args:
-        level: The log level to set.
-    """
-    if log.console_handler is None:
-        raise cmdutils.CommandError("No log.console_handler. Not attached "
-                                    "to a console?")
-
-    log.change_console_formatter(log.LOG_LEVELS[level.upper()])
-    log.console_handler.setLevel(log.LOG_LEVELS[level.upper()])
-
-
 @cmdutils.register(debug=True)
 def debug_log_filter(filters: str) -> None:
     """Change the log filter for console logging.
@@ -254,21 +236,17 @@ def debug_log_filter(filters: str) -> None:
         raise cmdutils.CommandError("No log.console_filter. Not attached "
                                     "to a console?")
 
-    if filters.strip().lower() == 'none':
-        log.console_filter.names = None
-        return
+    try:
+        new_filter = log.LogFilter.parse(filters)
+    except log.InvalidLogFilterError as e:
+        raise cmdutils.CommandError(e)
 
-    if not set(filters.split(',')).issubset(log.LOGGER_NAMES):
-        raise cmdutils.CommandError("filters: Invalid value {} - expected one "
-                                    "of: {}".format(
-                                        filters, ', '.join(log.LOGGER_NAMES)))
-
-    log.console_filter.names = filters.split(',')
+    log.console_filter.update_from(new_filter)
 
 
 @cmdutils.register()
 @cmdutils.argument('current_win_id', value=cmdutils.Value.win_id)
-def window_only(current_win_id):
+def window_only(current_win_id: int) -> None:
     """Close all windows except for the current one."""
     for win_id, window in objreg.window_registry.items():
 
@@ -282,7 +260,7 @@ def window_only(current_win_id):
 
 @cmdutils.register()
 @cmdutils.argument('win_id', value=cmdutils.Value.win_id)
-def version(win_id, paste=False):
+def version(win_id: int, paste: bool = False) -> None:
     """Show version information.
 
     Args:
@@ -294,3 +272,17 @@ def version(win_id, paste=False):
 
     if paste:
         pastebin_version()
+
+
+_keytester_widget = None  # type: typing.Optional[miscwidgets.KeyTesterWidget]
+
+
+@cmdutils.register(debug=True)
+def debug_keytester() -> None:
+    """Show a keytester widget."""
+    global _keytester_widget
+    if _keytester_widget and _keytester_widget.isVisible():
+        _keytester_widget.close()
+    else:
+        _keytester_widget = miscwidgets.KeyTesterWidget()
+        _keytester_widget.show()
